package com.muyu.vehicle.gateway.aliyun.service.impl;

import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.*;
import com.aliyun.tea.TeaException;
import com.aliyun.teautil.models.RuntimeOptions;
import com.muyu.common.redis.service.RedisService;
import com.muyu.vehicle.gateway.aliyun.domain.InstanceInformation;
import com.muyu.vehicle.gateway.aliyun.mapper.InstanceMapper;
import com.muyu.vehicle.gateway.aliyun.service.InstanceService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * ECS实例服务 实现类
 *
 * @author LiHD
 * @ClassName InstanceServiceImpl
 * @Date 2024/9/29 20:23
 */
@Log4j2
@Service
public class InstanceServiceImpl implements InstanceService {

    @Autowired
    private InstanceMapper instanceMapper;

    // 阿里云ECS相关参数
    private final static String AccessKeyId = "LTAI5tM4LSerzWQ9wUPXgybq"; // 请替换成您的 AccessKeyId
    private final static String AccessKeySecret = "hkiPr0QwkKMZauhXucgg1SDriXUW0Q"; // 请替换成您的 AccessKeySecret
    private final static String Endpoint = "ecs.cn-beijing.aliyuncs.com"; // 请替换成您的访问域名
    private final static String Size = "40"; // 请替换成您的实例系统盘大小，单位为GB
    private final static String Category = "cloud_essd"; // 请替换成您的实例系统盘类型，可选值：cloud_efficiency、cloud_ssd、cloud_essd
    private final static String RegionId = "cn-beijing"; // 请替换成您的实例所在地域ID
    private final static String ImageId = "m-2zehkv4epys91571e38r"; // 请替换成您的实例镜像ID
    private final static String InstanceType = "ecs.e-c1m1.large"; // 请替换成您的实例类型
    private final static String SecurityGroupId = "sg-2ze63acc0bas9vjm5gtv"; // 请替换成您的安全组ID
    private final static String VSwitchId = "vsw-2ze028qk4xpwk3i0ggket"; // 请替换成您的虚拟交换机ID
    private final static String InstanceName = "server-mqtt"; // 请替换成您的实例名称
    private final static String InstanceChargeType = "PostPaid"; // 请替换成您的实例付费类型，可选值：PrePaid、PostPaid
    private final static String HostName = "root"; // 请替换成您的主机名
    private final static String Password = "Lhd.123456"; // 请替换成您的实例密码
    private final static Integer Amount = 1; // 请替换成您要创建的实例数量
    private final static String InternetChargeType = "PayByTraffic"; // 请替换成您的实例网络计费类型，可选值：PayByBandwidth、PayByTraffic
    private final static Integer InternetMaxBandwidthOut = 5; // 请替换成您的实例网络出带宽，单位为Mbps
    @Autowired
    private RedisService redisService;

    /**
     * description
     * 用AK&amp;SK初始化账号Client
     * @returnClient
     * @throws Exception
     */
    public com.aliyun.ecs20140526.Client createClient() throws Exception {
        // 工程代码泄露可能会导致 AccessKey 泄露，并威胁账号下所有资源的安全性。以下代码示例仅供参考。
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config()
                // 必填，请确保代码运行环境设置了环境变量 ALIBABA_CLOUD_ACCESS_KEY_ID。
                .setAccessKeyId(AccessKeyId)
                // 必填，请确保代码运行环境设置了环境变量 ALIBABA_CLOUD_ACCESS_KEY_SECRET。
                .setAccessKeySecret(AccessKeySecret);
        // Endpoint 请参考 https://api.aliyun.com/product/Ecs
        config.endpoint = Endpoint;
        return new com.aliyun.ecs20140526.Client(config);
    }

    /**
     * 创建ECS实例
     */
    @Override
    public void createInstance(){
        log.info("创建ECS实例");
        // 创建阿里云ECS客户端
        Client client = null;
        try {
            client = this.createClient();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        // 配置系统盘参数
        com.aliyun.ecs20140526.models.RunInstancesRequest.RunInstancesRequestSystemDisk systemDisk =
                new com.aliyun.ecs20140526.models.RunInstancesRequest.RunInstancesRequestSystemDisk()
                .setSize(Size) // 设置系统盘大小
                .setCategory(Category);// 设置系统盘类型
        // 创建创建实例请求对象并设置参数
        com.aliyun.ecs20140526.models.RunInstancesRequest runInstancesRequest =
                new com.aliyun.ecs20140526.models.RunInstancesRequest()
                .setRegionId(RegionId) // 设置实例所在地域ID
                .setImageId(ImageId) // 设置实例镜像ID
                .setInstanceType(InstanceType) // 设置实例类型
                .setSecurityGroupId(SecurityGroupId) // 设置安全组ID
                .setVSwitchId(VSwitchId) // 设置虚拟交换机ID
                .setInstanceName(InstanceName) // 设置实例名称
                .setInstanceChargeType(InstanceChargeType) // 设置实例付费类型为后付费按量付费
                .setSystemDisk(systemDisk) // 设置系统盘配置
                .setHostName(HostName) // 设置主机名
                .setPassword(Password) // 设置实例密码
                .setAmount(Amount) // 设置创建实例的数量
                .setInternetChargeType(InternetChargeType)  // 设置互联网计费类型为按流量计费
                .setInternetMaxBandwidthOut(InternetMaxBandwidthOut);  // 设置最大出带宽为5Mbps

        // 创建运行时选项对象
        com.aliyun.teautil.models.RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();

        // 尝试执行创建实例请求
        try {
            // 复制代码运行请自行打印 API 的返回值
            RunInstancesResponse runInstancesResponse = client.runInstancesWithOptions(runInstancesRequest, runtime);
            RunInstancesResponseBody body = runInstancesResponse.getBody();
            ArrayList<String> list = new ArrayList<>(body.getInstanceIdSets().getInstanceIdSet());
            ArrayList<InstanceInformation> returnList = new ArrayList<>();
            log.info("创建的实例：{}", list);
            log.info("开始加载实例");
            Thread.sleep(30000);
            for (String s : list) {
                InstanceInformation instanceInformation = selectInstance(s);
                log.info("加载的实例：{}", instanceInformation);
                returnList.add(instanceInformation);
            }
            log.info("实例加载完成");
            redisService.setCacheObject("instanceList", returnList);

        } catch (TeaException error) {
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            log.info(error.getMessage());
            // 诊断地址
            log.info(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            log.info(error.getMessage());
            // 诊断地址
            log.info(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        }
    }

    /**
     * 清除ECS实例
     */
    @Override
    public void cleanInstance() {
        log.info("清理ECS实例");
        // 创建ECS客户端对象，用于后续调用ECS相关API
        Client client = null;
        try {
            client = this.createClient();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        DescribeInstancesRequest describeInstancesRequest = new DescribeInstancesRequest()
                .setRegionId(RegionId);

        //创建运行时选择对象,用于配置运行时的选项参数
        RuntimeOptions runtimeOptions = new RuntimeOptions();

        //获取实例列表
        DescribeInstancesResponse describeInstancesResponse = null;
        try {
            describeInstancesResponse = client.describeInstancesWithOptions(describeInstancesRequest, runtimeOptions);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        //提取实例ID集合
        ArrayList<String> list = new ArrayList<>();
        DescribeInstancesResponseBody body = describeInstancesResponse.getBody();

        for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance instance : body.getInstances().getInstance()) {
            list.add(instance.getInstanceId());
        }
        log.info("清除的实例ID：{}", list);
        // 创建删除实例请求对象，并设置请求参数
        com.aliyun.ecs20140526.models.DeleteInstancesRequest deleteInstancesRequest = new com.aliyun.ecs20140526.models.DeleteInstancesRequest()
                // 设置地域ID，指定删除实例的地域
                .setRegionId(RegionId)
                // 设置DryRun为true，用于验证请求是否可以成功，但不实际执行删除操作
                .setDryRun(false)
                // 设置Force为true，表示即使实例有正在运行的任务，也强制删除实例
                .setForce(true)
                // 设置TerminateSubscription为true，表示删除按订阅付费的实例时终止订阅
                .setTerminateSubscription(true)
                .setInstanceId(list);

        // 创建运行时选项对象，用于配置运行时的选项参数
        com.aliyun.teautil.models.RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();
        try {
            // 复制代码运行请自行打印 API 的返回值
            DeleteInstancesResponse deleteInstancesResponse = client.deleteInstancesWithOptions(deleteInstancesRequest, runtime);
        } catch (TeaException error) {
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            System.out.println(error.getMessage());
            // 诊断地址
            System.out.println(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            System.out.println(error.getMessage());
            // 诊断地址
            System.out.println(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        }
    }

    /**
     * 查询实例信息
     * @param instance
     * @return
     */
    public InstanceInformation selectInstance(String instance) {

        try {
            com.aliyun.ecs20140526.Client client = this.createClient();
            com.aliyun.ecs20140526.models.DescribeInstanceAttributeRequest describeInstanceAttributeRequest = new com.aliyun.ecs20140526.models.DescribeInstanceAttributeRequest()
                    .setInstanceId(instance);
            com.aliyun.teautil.models.RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();
            // 复制代码运行请自行打印 API 的返回值
            DescribeInstanceAttributeResponse describeInstanceAttributeResponse = client.describeInstanceAttributeWithOptions(describeInstanceAttributeRequest, runtime);
            // 获取返回值
            DescribeInstanceAttributeResponseBody body = describeInstanceAttributeResponse.getBody();
            // 获取公网IP地址
            List<String> ipAddress = body.publicIpAddress.ipAddress;
            //获取的实例ID
            String instanceId = body.instanceId;
            //获取的公网IP地址
            String ip = ipAddress.get(0);
            //获取服务器状态
            String status = body.status;
            InstanceInformation build = InstanceInformation.builder()
                    .Status(status)
                    .InstanceId(instanceId)
                    .IpAddress(ip).build();
            //返回获取的服务器信息
            return build;
        } catch (TeaException error) {
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            System.out.println(error.getMessage());
            // 诊断地址
            System.out.println(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            System.out.println(error.getMessage());
            // 诊断地址
            System.out.println(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        }
        return null;
    }
}
